# coding: utf-8

"""
Notifications API

API for notification delivery system

The version of the OpenAPI document: 1.0.0
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations

import json
import pprint
import re  # noqa: F401
from typing import Any, ClassVar, Dict, List, Optional, Set

from pydantic import BaseModel, ConfigDict, Field, StrictStr
from typing_extensions import Self

from crypticorn.notification.client.models.dashboard_action_button import (
    DashboardActionButton,
)
from crypticorn.notification.client.models.dashboard_field import DashboardField


class UITemplate(BaseModel):
    """
    UITemplate
    """  # noqa: E501

    title: StrictStr = Field(description="The title of the notification.")
    subtitle: Optional[StrictStr] = None
    body: StrictStr = Field(description="The body of the notification.")
    image_url: Optional[StrictStr] = Field(alias="imageUrl")
    actions: Optional[List[DashboardActionButton]]
    fields: Optional[List[DashboardField]]
    __properties: ClassVar[List[str]] = [
        "title",
        "subtitle",
        "body",
        "imageUrl",
        "actions",
        "fields",
    ]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of UITemplate from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in actions (list)
        _items = []
        if self.actions:
            for _item_actions in self.actions:
                if _item_actions:
                    _items.append(_item_actions.to_dict())
            _dict["actions"] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in fields (list)
        _items = []
        if self.fields:
            for _item_fields in self.fields:
                if _item_fields:
                    _items.append(_item_fields.to_dict())
            _dict["fields"] = _items
        # set to None if subtitle (nullable) is None
        # and model_fields_set contains the field
        if self.subtitle is None and "subtitle" in self.model_fields_set:
            _dict["subtitle"] = None

        # set to None if image_url (nullable) is None
        # and model_fields_set contains the field
        if self.image_url is None and "image_url" in self.model_fields_set:
            _dict["imageUrl"] = None

        # set to None if actions (nullable) is None
        # and model_fields_set contains the field
        if self.actions is None and "actions" in self.model_fields_set:
            _dict["actions"] = None

        # set to None if fields (nullable) is None
        # and model_fields_set contains the field
        if self.fields is None and "fields" in self.model_fields_set:
            _dict["fields"] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of UITemplate from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "title": obj.get("title"),
                "subtitle": obj.get("subtitle"),
                "body": obj.get("body"),
                "imageUrl": obj.get("imageUrl"),
                "actions": (
                    [DashboardActionButton.from_dict(_item) for _item in obj["actions"]]
                    if obj.get("actions") is not None
                    else None
                ),
                "fields": (
                    [DashboardField.from_dict(_item) for _item in obj["fields"]]
                    if obj.get("fields") is not None
                    else None
                ),
            }
        )
        return _obj



# coding: utf-8

"""
Payment API

API for accepting payments and storing subscriptions

The version of the OpenAPI document: 1.0.0
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations

import json
import pprint
import re  # noqa: F401
from typing import Any, ClassVar, Dict, List, Optional, Set

from pydantic import (
    BaseModel,
    ConfigDict,
    Field,
    StrictBool,
    StrictInt,
    StrictStr,
    field_validator,
)
from typing_extensions import Self

from crypticorn.pay.client.models.scope import Scope


class ScopeInfo(BaseModel):
    """
    Model for detailed scope access info for a user, for each access method.
    """  # noqa: E501

    scope: Scope = Field(description="The scope affected")
    expires_at: Optional[StrictInt] = None
    has_expired: StrictBool = Field(description="Whether the scope has expired or not")
    reason: StrictStr = Field(
        description="Reason for access (allowlist, subscription, balance, etc.)"
    )
    __properties: ClassVar[List[str]] = ["scope", "expires_at", "has_expired", "reason"]

    @field_validator("reason")
    def reason_validate_enum(cls, value):
        """Validates the enum"""
        if value not in set(["allowlist", "subscription", "balance"]):
            raise ValueError(
                "must be one of enum values ('allowlist', 'subscription', 'balance')"
            )
        return value

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of ScopeInfo from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # set to None if expires_at (nullable) is None
        # and model_fields_set contains the field
        if self.expires_at is None and "expires_at" in self.model_fields_set:
            _dict["expires_at"] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of ScopeInfo from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "scope": obj.get("scope"),
                "expires_at": obj.get("expires_at"),
                "has_expired": obj.get("has_expired"),
                "reason": obj.get("reason"),
            }
        )
        return _obj



import logging
from pathlib import Path

import requests
import tqdm

logger = logging.getLogger("crypticorn")


def download_file(url: str, dest_path: Path, show_progress_bars: bool = True) -> Path:
    """downloads a file and shows a progress bar. allow resuming a download"""
    file_size = 0
    req = requests.get(url, stream=True, timeout=600)
    req.raise_for_status()

    total_size = int(req.headers.get("content-length", 0))
    temp_path = dest_path.joinpath(".temp")

    if dest_path.exists():
        logger.info(f" file already exists: {dest_path}")
        file_size = dest_path.stat().st_size
        if file_size == total_size:
            return dest_path

    if temp_path.exists():
        file_size = temp_path.stat().st_size

        if file_size < total_size:
            # Download incomplete
            logger.info(" resuming download")
            resume_header = {"Range": f"bytes={file_size}-"}
            req = requests.get(
                url,
                headers=resume_header,
                stream=True,
                verify=False,
                allow_redirects=True,
                timeout=600,
            )
        else:
            # Error, delete file and restart download
            logger.error(f" deleting file {dest_path} and restarting")
            temp_path.unlink()
            file_size = 0
    else:
        # File does not exist, starting download
        logger.info(" starting download")

    # write dataset to file and show progress bar
    pbar = tqdm.tqdm(
        total=total_size,
        unit="B",
        unit_scale=True,
        desc=dest_path,
        disable=not show_progress_bars,
    )
    # Update progress bar to reflect how much of the file is already downloaded
    pbar.update(file_size)
    with temp_path.open("ab") as dest_file:
        for chunk in req.iter_content(1024):
            dest_file.write(chunk)
            pbar.update(1024)
    # move temp file to target destination
    temp_path.replace(dest_path)
    return dest_path



# coding: utf-8

"""
Trading API

API for automated trading and exchange interface. This API is used to trade on the exchange and manage bots, API keys, orders, and more.

The version of the OpenAPI document: 2.0.0
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations

import json
import pprint
import re  # noqa: F401
from typing import Any, ClassVar, Dict, List, Optional, Set, Union

from pydantic import (
    BaseModel,
    ConfigDict,
    Field,
    StrictBool,
    StrictFloat,
    StrictInt,
    StrictStr,
)
from typing_extensions import Self


class VaultDetails(BaseModel):
    """
    Model for vault details
    """  # noqa: E501

    name: StrictStr = Field(description="Vault name")
    vault_address: StrictStr = Field(description="Vault address")
    leader: StrictStr = Field(description="Vault leader address")
    description: StrictStr = Field(description="Vault description")
    apr: Union[StrictFloat, StrictInt] = Field(description="Annual percentage rate")
    leader_fraction: Union[StrictFloat, StrictInt] = Field(
        description="Leader fraction"
    )
    leader_commission: Union[StrictFloat, StrictInt] = Field(
        description="Leader commission"
    )
    max_distributable: Union[StrictFloat, StrictInt] = Field(
        description="Maximum distributable amount"
    )
    max_withdrawable: Union[StrictFloat, StrictInt] = Field(
        description="Maximum withdrawable amount"
    )
    is_closed: StrictBool = Field(description="Whether the vault is closed")
    allow_deposits: StrictBool = Field(description="Whether deposits are allowed")
    tvl: Union[StrictFloat, StrictInt] = Field(description="Current TVL of the vault")
    __properties: ClassVar[List[str]] = [
        "name",
        "vault_address",
        "leader",
        "description",
        "apr",
        "leader_fraction",
        "leader_commission",
        "max_distributable",
        "max_withdrawable",
        "is_closed",
        "allow_deposits",
        "tvl",
    ]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of VaultDetails from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of VaultDetails from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "name": obj.get("name"),
                "vault_address": obj.get("vault_address"),
                "leader": obj.get("leader"),
                "description": obj.get("description"),
                "apr": obj.get("apr"),
                "leader_fraction": obj.get("leader_fraction"),
                "leader_commission": obj.get("leader_commission"),
                "max_distributable": obj.get("max_distributable"),
                "max_withdrawable": obj.get("max_withdrawable"),
                "is_closed": obj.get("is_closed"),
                "allow_deposits": obj.get("allow_deposits"),
                "tvl": obj.get("tvl"),
            }
        )
        return _obj



# coding: utf-8

# flake8: noqa

"""
DEX AI API

API for DEX AI

The version of the OpenAPI document: 1.0.0
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501


__version__ = "1.0.0"

# Define package exports
__all__ = [
    "SignalsApi",
    "StatusApi",
    "ApiResponse",
    "ApiClient",
    "Configuration",
    "OpenApiException",
    "ApiTypeError",
    "ApiValueError",
    "ApiKeyError",
    "ApiAttributeError",
    "ApiException",
    "ApiErrorIdentifier",
    "ApiErrorLevel",
    "ApiErrorType",
    "ExceptionDetail",
    "PaginatedResponseSignalWithToken",
    "Risk",
    "SignalOverviewStats",
    "SignalVolume",
    "SignalWithToken",
    "TokenData",
    "TokenDetail",
]

# import apis into sdk package
from crypticorn.dex.client.api.signals_api import SignalsApi as SignalsApi
from crypticorn.dex.client.api.status_api import StatusApi as StatusApi
from crypticorn.dex.client.api_client import ApiClient as ApiClient

# import ApiClient
from crypticorn.dex.client.api_response import ApiResponse as ApiResponse
from crypticorn.dex.client.configuration import Configuration as Configuration
from crypticorn.dex.client.exceptions import ApiAttributeError as ApiAttributeError
from crypticorn.dex.client.exceptions import ApiException as ApiException
from crypticorn.dex.client.exceptions import ApiKeyError as ApiKeyError
from crypticorn.dex.client.exceptions import ApiTypeError as ApiTypeError
from crypticorn.dex.client.exceptions import ApiValueError as ApiValueError
from crypticorn.dex.client.exceptions import OpenApiException as OpenApiException

# import models into sdk package
from crypticorn.dex.client.models.api_error_identifier import (
    ApiErrorIdentifier as ApiErrorIdentifier,
)
from crypticorn.dex.client.models.api_error_level import ApiErrorLevel as ApiErrorLevel
from crypticorn.dex.client.models.api_error_type import ApiErrorType as ApiErrorType
from crypticorn.dex.client.models.exception_detail import (
    ExceptionDetail as ExceptionDetail,
)
from crypticorn.dex.client.models.paginated_response_signal_with_token import (
    PaginatedResponseSignalWithToken as PaginatedResponseSignalWithToken,
)
from crypticorn.dex.client.models.risk import Risk as Risk
from crypticorn.dex.client.models.signal_overview_stats import (
    SignalOverviewStats as SignalOverviewStats,
)
from crypticorn.dex.client.models.signal_volume import SignalVolume as SignalVolume
from crypticorn.dex.client.models.signal_with_token import (
    SignalWithToken as SignalWithToken,
)
from crypticorn.dex.client.models.token_data import TokenData as TokenData
from crypticorn.dex.client.models.token_detail import TokenDetail as TokenDetail



# coding: utf-8

"""
Crypticorn Auth API

OpenAPI compliant REST API built using tRPC with Express

The version of the OpenAPI document: 1.0.0
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations

import json
import pprint
import re  # noqa: F401
from typing import Any, ClassVar, Dict, List, Optional, Set

from pydantic import BaseModel, ConfigDict, Field, StrictStr
from typing_extensions import Self


class RefreshTokenInfo200ResponseUserSession(BaseModel):
    """
    RefreshTokenInfo200ResponseUserSession
    """  # noqa: E501

    id: StrictStr = Field(description="ID of the user session")
    user_id: StrictStr = Field(description="User ID of the user session")
    token: StrictStr = Field(description="Token of the user session")
    expires_at: StrictStr = Field(description="Expiration time of the user session")
    client_ip: Optional[StrictStr] = Field(
        description="Client IP address of the user session"
    )
    user_agent: Optional[StrictStr] = Field(
        description="User agent of the user session"
    )
    __properties: ClassVar[List[str]] = [
        "id",
        "user_id",
        "token",
        "expires_at",
        "client_ip",
        "user_agent",
    ]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of RefreshTokenInfo200ResponseUserSession from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # set to None if client_ip (nullable) is None
        # and model_fields_set contains the field
        if self.client_ip is None and "client_ip" in self.model_fields_set:
            _dict["client_ip"] = None

        # set to None if user_agent (nullable) is None
        # and model_fields_set contains the field
        if self.user_agent is None and "user_agent" in self.model_fields_set:
            _dict["user_agent"] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of RefreshTokenInfo200ResponseUserSession from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "id": obj.get("id"),
                "user_id": obj.get("user_id"),
                "token": obj.get("token"),
                "expires_at": obj.get("expires_at"),
                "client_ip": obj.get("client_ip"),
                "user_agent": obj.get("user_agent"),
            }
        )
        return _obj



# coding: utf-8

"""
Trading API

API for automated trading and exchange interface. This API is used to trade on the exchange and manage bots, API keys, orders, and more.

The version of the OpenAPI document: 2.0.0
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations

import json
import pprint
import re  # noqa: F401
from typing import Any, ClassVar, Dict, List, Optional, Set, Union

from pydantic import BaseModel, ConfigDict, Field, StrictFloat, StrictInt
from typing_extensions import Self


class PnL(BaseModel):
    """
    The profit and loss of a bot by timestamp. In the case of sampling, the PnL is the sum of the PnLs between the prior timestamp and the current timestamp.
    """  # noqa: E501

    timestamp: StrictInt = Field(description="Timestamp of the order")
    pnl: Union[StrictFloat, StrictInt] = Field(
        description="The profit and loss, either absolute or relative depending on the request"
    )
    cum_pnl: Union[StrictFloat, StrictInt] = Field(
        description="The cumulative profit and loss of the bot until the order (inclusive), either absolute or relative depending on the request"
    )
    __properties: ClassVar[List[str]] = ["timestamp", "pnl", "cum_pnl"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of PnL from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of PnL from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "timestamp": obj.get("timestamp"),
                "pnl": obj.get("pnl"),
                "cum_pnl": obj.get("cum_pnl"),
            }
        )
        return _obj



# coding: utf-8

# flake8: noqa
"""
Hive AI API

API for Hive AI model training and evaluation

The version of the OpenAPI document: 1.0.0
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501


# import models into model package
from crypticorn.hive.client.models.api_error_identifier import ApiErrorIdentifier
from crypticorn.hive.client.models.api_error_level import ApiErrorLevel
from crypticorn.hive.client.models.api_error_type import ApiErrorType
from crypticorn.hive.client.models.coin_info import CoinInfo
from crypticorn.hive.client.models.coins import Coins
from crypticorn.hive.client.models.data_download_response import DataDownloadResponse
from crypticorn.hive.client.models.data_info import DataInfo
from crypticorn.hive.client.models.data_options import DataOptions
from crypticorn.hive.client.models.data_version import DataVersion
from crypticorn.hive.client.models.data_version_info import DataVersionInfo
from crypticorn.hive.client.models.download_links import DownloadLinks
from crypticorn.hive.client.models.evaluation import Evaluation
from crypticorn.hive.client.models.evaluation_response import EvaluationResponse
from crypticorn.hive.client.models.exception_detail import ExceptionDetail
from crypticorn.hive.client.models.feature_size import FeatureSize
from crypticorn.hive.client.models.model_create import ModelCreate
from crypticorn.hive.client.models.model_read import ModelRead
from crypticorn.hive.client.models.model_status import ModelStatus
from crypticorn.hive.client.models.model_update import ModelUpdate
from crypticorn.hive.client.models.target import Target
from crypticorn.hive.client.models.target_info import TargetInfo
from crypticorn.hive.client.models.target_type import TargetType



from __future__ import annotations

import os
from pathlib import Path
from typing import TYPE_CHECKING, Awaitable, Optional, Union

from pydantic import StrictInt

from crypticorn.hive import (
    ApiClient,
    Configuration,
    DataApi,
    DataVersion,
    FeatureSize,
    ModelsApi,
    StatusApi,
)
from crypticorn.hive.utils import download_file

if TYPE_CHECKING:
    from aiohttp import ClientSession


class HiveClient:
    """
    A client for interacting with the Crypticorn Hive API.
    """

    config_class = Configuration

    def __init__(
        self,
        config: Configuration,
        http_client: Optional[ClientSession] = None,
        is_sync: bool = False,
    ):
        self.config = config
        self.base_client = ApiClient(configuration=self.config)
        if http_client is not None:
            self.base_client.rest_client.pool_manager = http_client
        # Pass sync context to REST client for proper session management
        self.base_client.rest_client.is_sync = is_sync
        # Instantiate all the endpoint clients
        self.models = ModelsApi(self.base_client, is_sync=is_sync)
        self.data = DataApiWrapper(self.base_client, is_sync=is_sync)
        self.status = StatusApi(self.base_client, is_sync=is_sync)


class DataApiWrapper(DataApi):
    """
    A wrapper for the DataApi class.
    """

    def download_data(  # type: ignore[override]
        self,
        model_id: StrictInt,
        folder: Path = Path("data"),
        version: Optional[DataVersion] = None,
        feature_size: Optional[FeatureSize] = None,
        **kwargs,
    ) -> Union[list[Path], Awaitable[list[Path]]]:
        """
        Download data for model training. All three files (y_train, x_test, x_train) are downloaded and saved under e.g. FOLDER/v1/coin_1/*.feather.
        The folder will be created if it doesn't exist.
        Works in both sync and async contexts.

        :param model_id: Model ID (required) (type: int)
        :param version: Data version. Default is the latest public version. (optional) (type: DataVersion)
        :param feature_size: The number of features in the data. Default is LARGE. (optional) (type: FeatureSize)
        :return: A list of paths to the downloaded files.
        """
        if self.is_sync:
            return self._download_data_wrapper_sync(
                model_id=model_id,
                folder=folder,
                version=version,
                feature_size=feature_size,
                **kwargs,
            )
        else:
            return self._download_data_wrapper_async(
                model_id=model_id,
                folder=folder,
                version=version,
                feature_size=feature_size,
                **kwargs,
            )

    def _download_data_wrapper_sync(
        self,
        model_id: StrictInt,
        folder: Path = Path("data"),
        version: Optional[DataVersion] = None,
        feature_size: Optional[FeatureSize] = None,
        **kwargs,
    ) -> list[Path]:
        """
        Download data for model training (sync version).
        """
        response = self._download_data_sync(
            model_id=model_id,
            version=version,
            feature_size=feature_size,
            **kwargs,
        )
        base_path = f"{folder}/v{response.version.value}/coin_{response.coin.value}/"
        os.makedirs(base_path, exist_ok=True)

        return [
            download_file(
                url=response.links.y_train,
                dest_path=Path(base_path + "y_train_" + response.target + ".feather"),
            ),
            download_file(
                url=response.links.x_test,
                dest_path=Path(
                    base_path + "x_test_" + response.feature_size + ".feather"
                ),
            ),
            download_file(
                url=response.links.x_train,
                dest_path=Path(
                    base_path + "x_train_" + response.feature_size + ".feather"
                ),
            ),
        ]

    async def _download_data_wrapper_async(
        self,
        model_id: StrictInt,
        folder: Path = Path("data"),
        version: Optional[DataVersion] = None,
        feature_size: Optional[FeatureSize] = None,
        **kwargs,
    ) -> list[Path]:
        """
        Download data for model training (async version).
        """
        response = await self._download_data_async(
            model_id=model_id,
            version=version,
            feature_size=feature_size,
            **kwargs,
        )
        base_path = f"{folder}/v{response.version.value}/coin_{response.coin.value}/"
        os.makedirs(base_path, exist_ok=True)

        return [
            download_file(
                url=response.links.y_train,
                dest_path=Path(base_path + "y_train_" + response.target + ".feather"),
            ),
            download_file(
                url=response.links.x_test,
                dest_path=Path(
                    base_path + "x_test_" + response.feature_size + ".feather"
                ),
            ),
            download_file(
                url=response.links.x_train,
                dest_path=Path(
                    base_path + "x_train_" + response.feature_size + ".feather"
                ),
            ),
        ]



# coding: utf-8

# flake8: noqa

"""
Marketcap Service API

API for retrieving historical marketcap data, available exchanges, and indicators.

The version of the OpenAPI document: 1.0.0
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501


__version__ = "1.0.0"

# Define package exports
__all__ = [
    "ExchangesApi",
    "IndicatorsApi",
    "LogsApi",
    "MarketcapApi",
    "MarketsApi",
    "QuoteCurrenciesApi",
    "StatusApi",
    "TokensApi",
    "ApiResponse",
    "ApiClient",
    "Configuration",
    "OpenApiException",
    "ApiTypeError",
    "ApiValueError",
    "ApiKeyError",
    "ApiAttributeError",
    "ApiException",
    "ApiErrorIdentifier",
    "ApiErrorLevel",
    "ApiErrorType",
    "ExceptionDetail",
    "ExchangeAvailability",
    "ExchangeMapping",
    "InternalExchange",
    "MarketType",
    "MarketcapRanking",
    "MarketcapSymbolRanking",
    "OHLCV",
    "Severity",
    "TimeInterval",
    "TradingStatus",
]

# import apis into sdk package
from crypticorn.metrics.client.api.exchanges_api import ExchangesApi as ExchangesApi
from crypticorn.metrics.client.api.indicators_api import IndicatorsApi as IndicatorsApi
from crypticorn.metrics.client.api.logs_api import LogsApi as LogsApi
from crypticorn.metrics.client.api.marketcap_api import MarketcapApi as MarketcapApi
from crypticorn.metrics.client.api.markets_api import MarketsApi as MarketsApi
from crypticorn.metrics.client.api.quote_currencies_api import (
    QuoteCurrenciesApi as QuoteCurrenciesApi,
)
from crypticorn.metrics.client.api.status_api import StatusApi as StatusApi
from crypticorn.metrics.client.api.tokens_api import TokensApi as TokensApi
from crypticorn.metrics.client.api_client import ApiClient as ApiClient

# import ApiClient
from crypticorn.metrics.client.api_response import ApiResponse as ApiResponse
from crypticorn.metrics.client.configuration import Configuration as Configuration
from crypticorn.metrics.client.exceptions import ApiAttributeError as ApiAttributeError
from crypticorn.metrics.client.exceptions import ApiException as ApiException
from crypticorn.metrics.client.exceptions import ApiKeyError as ApiKeyError
from crypticorn.metrics.client.exceptions import ApiTypeError as ApiTypeError
from crypticorn.metrics.client.exceptions import ApiValueError as ApiValueError
from crypticorn.metrics.client.exceptions import OpenApiException as OpenApiException

# import models into sdk package
from crypticorn.metrics.client.models.api_error_identifier import (
    ApiErrorIdentifier as ApiErrorIdentifier,
)
from crypticorn.metrics.client.models.api_error_level import (
    ApiErrorLevel as ApiErrorLevel,
)
from crypticorn.metrics.client.models.api_error_type import ApiErrorType as ApiErrorType
from crypticorn.metrics.client.models.exception_detail import (
    ExceptionDetail as ExceptionDetail,
)
from crypticorn.metrics.client.models.exchange_availability import (
    ExchangeAvailability as ExchangeAvailability,
)
from crypticorn.metrics.client.models.exchange_mapping import (
    ExchangeMapping as ExchangeMapping,
)
from crypticorn.metrics.client.models.internal_exchange import (
    InternalExchange as InternalExchange,
)
from crypticorn.metrics.client.models.market_type import MarketType as MarketType
from crypticorn.metrics.client.models.marketcap_ranking import (
    MarketcapRanking as MarketcapRanking,
)
from crypticorn.metrics.client.models.marketcap_symbol_ranking import (
    MarketcapSymbolRanking as MarketcapSymbolRanking,
)
from crypticorn.metrics.client.models.ohlcv import OHLCV as OHLCV
from crypticorn.metrics.client.models.severity import Severity as Severity
from crypticorn.metrics.client.models.time_interval import TimeInterval as TimeInterval
from crypticorn.metrics.client.models.trading_status import (
    TradingStatus as TradingStatus,
)



# coding: utf-8

# flake8: noqa

"""
Trading API

API for automated trading and exchange interface. This API is used to trade on the exchange and manage bots, API keys, orders, and more.

The version of the OpenAPI document: 2.0.0
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501


__version__ = "1.0.0"

# Define package exports
__all__ = [
    "APIKeysApi",
    "BotsApi",
    "StatusApi",
    "StrategiesApi",
    "TradingActionsApi",
    "ApiResponse",
    "ApiClient",
    "Configuration",
    "OpenApiException",
    "ApiTypeError",
    "ApiValueError",
    "ApiKeyError",
    "ApiAttributeError",
    "ApiException",
    "Bot",
    "BotCreate",
    "BotUpdate",
    "ExchangeKey",
    "ExchangeKeyCreate",
    "ExchangeKeyUpdate",
    "ExecutionIds",
    "FuturesTradingAction",
    "FuturesTradingActionCreate",
    "Order",
    "OrdersCount",
    "PaginatedResponseFuturesTradingAction",
    "PaginatedResponseOrder",
    "PnL",
    "PostFuturesAction",
    "SpotTradingActionCreate",
    "Strategy",
    "StrategyCreate",
    "StrategyUpdate",
    "TPSL",
    "TPSLCreate",
    "TradingActionType",
    "VaultBot",
    "VaultDetails",
]

# import apis into sdk package
from crypticorn.trade.client.api.api_keys_api import APIKeysApi as APIKeysApi
from crypticorn.trade.client.api.bots_api import BotsApi as BotsApi
from crypticorn.trade.client.api.status_api import StatusApi as StatusApi
from crypticorn.trade.client.api.strategies_api import StrategiesApi as StrategiesApi
from crypticorn.trade.client.api.trading_actions_api import (
    TradingActionsApi as TradingActionsApi,
)
from crypticorn.trade.client.api_client import ApiClient as ApiClient

# import ApiClient
from crypticorn.trade.client.api_response import ApiResponse as ApiResponse
from crypticorn.trade.client.configuration import Configuration as Configuration
from crypticorn.trade.client.exceptions import ApiAttributeError as ApiAttributeError
from crypticorn.trade.client.exceptions import ApiException as ApiException
from crypticorn.trade.client.exceptions import ApiKeyError as ApiKeyError
from crypticorn.trade.client.exceptions import ApiTypeError as ApiTypeError
from crypticorn.trade.client.exceptions import ApiValueError as ApiValueError
from crypticorn.trade.client.exceptions import OpenApiException as OpenApiException

# import models into sdk package
from crypticorn.trade.client.models.bot import Bot as Bot
from crypticorn.trade.client.models.bot_create import BotCreate as BotCreate
from crypticorn.trade.client.models.bot_update import BotUpdate as BotUpdate
from crypticorn.trade.client.models.exchange_key import ExchangeKey as ExchangeKey
from crypticorn.trade.client.models.exchange_key_create import (
    ExchangeKeyCreate as ExchangeKeyCreate,
)
from crypticorn.trade.client.models.exchange_key_update import (
    ExchangeKeyUpdate as ExchangeKeyUpdate,
)
from crypticorn.trade.client.models.execution_ids import ExecutionIds as ExecutionIds
from crypticorn.trade.client.models.futures_trading_action import (
    FuturesTradingAction as FuturesTradingAction,
)
from crypticorn.trade.client.models.futures_trading_action_create import (
    FuturesTradingActionCreate as FuturesTradingActionCreate,
)
from crypticorn.trade.client.models.order import Order as Order
from crypticorn.trade.client.models.orders_count import OrdersCount as OrdersCount
from crypticorn.trade.client.models.paginated_response_futures_trading_action import (
    PaginatedResponseFuturesTradingAction as PaginatedResponseFuturesTradingAction,
)
from crypticorn.trade.client.models.paginated_response_order import (
    PaginatedResponseOrder as PaginatedResponseOrder,
)
from crypticorn.trade.client.models.pn_l import PnL as PnL
from crypticorn.trade.client.models.post_futures_action import (
    PostFuturesAction as PostFuturesAction,
)
from crypticorn.trade.client.models.spot_trading_action_create import (
    SpotTradingActionCreate as SpotTradingActionCreate,
)
from crypticorn.trade.client.models.strategy import Strategy as Strategy
from crypticorn.trade.client.models.strategy_create import (
    StrategyCreate as StrategyCreate,
)
from crypticorn.trade.client.models.strategy_update import (
    StrategyUpdate as StrategyUpdate,
)
from crypticorn.trade.client.models.tpsl import TPSL as TPSL
from crypticorn.trade.client.models.tpsl_create import TPSLCreate as TPSLCreate
from crypticorn.trade.client.models.trading_action_type import (
    TradingActionType as TradingActionType,
)
from crypticorn.trade.client.models.vault_bot import VaultBot as VaultBot
from crypticorn.trade.client.models.vault_details import VaultDetails as VaultDetails



# coding: utf-8

"""
Payment API

API for accepting payments and storing subscriptions

The version of the OpenAPI document: 1.0.0
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations

import json
import pprint
import re  # noqa: F401
from typing import Any, ClassVar, Dict, List, Optional, Set, Union

from pydantic import (
    BaseModel,
    ConfigDict,
    Field,
    StrictFloat,
    StrictInt,
    StrictStr,
    field_validator,
)
from typing_extensions import Self


class Payment(BaseModel):
    """
    Combined payment model across all services
    """  # noqa: E501

    id: StrictStr = Field(description="Payment ID")
    product_id: StrictStr = Field(description="Product purchased")
    coupon_id: Optional[StrictStr] = None
    user_id: StrictStr = Field(description="User ID the payment is for")
    invoice_id: StrictStr = Field(description="Invoice ID")
    amount: Union[StrictFloat, StrictInt] = Field(description="Payment amount")
    currency: StrictStr = Field(description="Payment currency")
    status: StrictStr = Field(description="Payment status")
    provider: StrictStr = Field(description="Payment provider")
    market: StrictStr = Field(description="Payment market")
    updated_at: StrictInt = Field(description="Payment updated at timestamp in seconds")
    created_at: StrictInt = Field(description="Payment created at timestamp in seconds")
    details: Optional[Dict[str, Any]] = Field(
        default=None, description="Payment details specific to the provider"
    )
    __properties: ClassVar[List[str]] = [
        "id",
        "product_id",
        "coupon_id",
        "user_id",
        "invoice_id",
        "amount",
        "currency",
        "status",
        "provider",
        "market",
        "updated_at",
        "created_at",
        "details",
    ]

    @field_validator("status")
    def status_validate_enum(cls, value):
        """Validates the enum"""
        if value not in set(
            [
                "waiting",
                "processing",
                "paid",
                "partially_paid",
                "refunded",
                "failed",
                "cancelled",
            ]
        ):
            raise ValueError(
                "must be one of enum values ('waiting', 'processing', 'paid', 'partially_paid', 'refunded', 'failed', 'cancelled')"
            )
        return value

    @field_validator("provider")
    def provider_validate_enum(cls, value):
        """Validates the enum"""
        if value not in set(["now", "staking", "stripe"]):
            raise ValueError("must be one of enum values ('now', 'staking', 'stripe')")
        return value

    @field_validator("market")
    def market_validate_enum(cls, value):
        """Validates the enum"""
        if value not in set(["crypto", "fiat"]):
            raise ValueError("must be one of enum values ('crypto', 'fiat')")
        return value

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of Payment from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # set to None if coupon_id (nullable) is None
        # and model_fields_set contains the field
        if self.coupon_id is None and "coupon_id" in self.model_fields_set:
            _dict["coupon_id"] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of Payment from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "id": obj.get("id"),
                "product_id": obj.get("product_id"),
                "coupon_id": obj.get("coupon_id"),
                "user_id": obj.get("user_id"),
                "invoice_id": obj.get("invoice_id"),
                "amount": obj.get("amount"),
                "currency": obj.get("currency"),
                "status": obj.get("status"),
                "provider": obj.get("provider"),
                "market": obj.get("market"),
                "updated_at": obj.get("updated_at"),
                "created_at": obj.get("created_at"),
                "details": obj.get("details"),
            }
        )
        return _obj



from importlib.metadata import version
from typing import Literal, Optional, TypeVar

from aiohttp import ClientSession, ClientTimeout, TCPConnector

from crypticorn.auth import AuthClient
from crypticorn.dex import DexClient
from crypticorn.hive import HiveClient
from crypticorn.metrics import MetricsClient
from crypticorn.notification import NotificationClient
from crypticorn.pay import PayClient
from crypticorn.trade import TradeClient

ConfigT = TypeVar("ConfigT")
SubClient = TypeVar("SubClient")
_SERVICES = Literal["hive", "trade", "pay", "metrics", "auth", "dex", "notification"]


class BaseAsyncClient:
    """
    Base class for Crypticorn API clients containing shared functionality.
    """

    def __init__(
        self,
        *,
        api_key: Optional[str] = None,
        jwt: Optional[str] = None,
        base_url: Optional[str] = None,
        is_sync: bool = False,
        http_client: Optional[ClientSession] = None,
    ):
        """
        :param api_key: The API key to use for authentication (recommended).
        :param jwt: The JWT to use for authentication (not recommended).
        :param base_url: The base URL the client will use to connect to the API.
        :param is_sync: Whether this client should operate in synchronous mode.
        :param http_client: Optional aiohttp ClientSession to use for HTTP requests.
        """
        self._base_url = (
            base_url.rstrip("/") if base_url else "https://api.crypticorn.com"
        )
        self._api_key = api_key
        self._jwt = jwt
        self._is_sync = is_sync
        self._http_client = http_client
        self._owns_http_client = http_client is None  # whether we own the http client

        self._service_classes: dict[_SERVICES, tuple[SubClient, str]] = {
            "hive": (HiveClient, "v1/hive"),
            "trade": (TradeClient, "v2/trade"),
            "pay": (PayClient, "v1/pay"),
            "metrics": (MetricsClient, "v1/metrics"),
            "auth": (AuthClient, "v1/auth"),
            "dex": (DexClient, "v1/dex"),
            "notification": (NotificationClient, "v1/notification"),
        }

        self._services = self._create_services()

    def _create_services(self) -> dict[_SERVICES, SubClient]:
        """Create services with the appropriate configuration based on sync/async mode."""
        services = {}
        for name, (client_class, path) in self._service_classes.items():
            config = self._get_default_config(client_class, path)
            # For sync clients, don't pass the persistent http_client
            # Let each operation manage its own session
            if self._is_sync:
                services[name] = client_class(
                    config, http_client=None, is_sync=self._is_sync
                )
            else:
                services[name] = client_class(
                    config, http_client=self._http_client, is_sync=self._is_sync
                )
        return services

    @property
    def base_url(self) -> str:
        """
        The base URL the client will use to connect to the API.
        """
        return self._base_url

    @property
    def api_key(self) -> Optional[str]:
        """
        The API key the client will use to connect to the API.
        This is the preferred way to authenticate.
        """
        return self._api_key

    @property
    def jwt(self) -> Optional[str]:
        """
        The JWT the client will use to connect to the API.
        This is the not the preferred way to authenticate.
        """
        return self._jwt

    @property
    def version(self) -> str:
        """
        The version of the client.
        """
        return version("crypticorn")

    @property
    def is_sync(self) -> bool:
        """
        Whether this client operates in synchronous mode.
        """
        return self._is_sync

    @property
    def http_client(self) -> Optional[ClientSession]:
        """
        The HTTP client session being used, if any.
        """
        return self._http_client

    @property
    def hive(self) -> HiveClient:
        """
        Entry point for the Hive AI API ([Docs](https://docs.crypticorn.com/api/?api=hive-ai-api)).
        """
        return self._services["hive"]

    @property
    def trade(self) -> TradeClient:
        """
        Entry point for the Trading API ([Docs](https://docs.crypticorn.com/api/?api=trading-api)).
        """
        return self._services["trade"]

    @property
    def metrics(self) -> MetricsClient:
        """
        Entry point for the Metrics API ([Docs](https://docs.crypticorn.com/api/?api=metrics-api)).
        """
        return self._services["metrics"]

    @property
    def pay(self) -> PayClient:
        """
        Entry point for the Payment API ([Docs](https://docs.crypticorn.com/api/?api=payment-api)).
        """
        return self._services["pay"]

    @property
    def auth(self) -> AuthClient:
        """
        Entry point for the Auth API ([Docs](https://docs.crypticorn.com/api/?api=auth-api)).
        """
        return self._services["auth"]

    @property
    def notification(self) -> NotificationClient:
        """
        Entry point for the Notification API ([Docs](https://docs.crypticorn.com/api/?api=notification-api)).
        """
        return self._services["notification"]

    @property
    def dex(self) -> DexClient:
        """
        Entry point for the DEX API ([Docs](https://docs.crypticorn.com/api/?api=dex-api)).
        """
        return self._services["dex"]

    def configure(self, config: ConfigT, service: _SERVICES) -> None:
        """
        Update a sub-client's configuration by overriding with the values set in the new config.
        Useful for testing a specific service against a local server instead of the default proxy.

        :param config: The new configuration to use for the sub-client.
        :param service: The service to configure.

        Example:
        >>> # For async client
        >>> async with AsyncClient() as client:
        ...     client.configure(config=HiveConfig(host="http://localhost:8000"), service='hive')
        >>>
        >>> # For sync client
        >>> with SyncClient() as client:
        ...     client.configure(config=HiveConfig(host="http://localhost:8000"), service='hive')
        """
        assert (
            service in self._service_classes
        ), f"Invalid service: {service}. Must be one of {list(self._service_classes.keys())}"
        client = self._services[service]
        new_config = client.config
        for attr in vars(config):
            new_value = getattr(config, attr)
            if new_value:
                setattr(new_config, attr, new_value)

        # Recreate service with new config and appropriate parameters
        client_class = type(client)
        if self._is_sync:
            self._services[service] = client_class(
                new_config, is_sync=self._is_sync, http_client=self._http_client
            )
        else:
            self._services[service] = client_class(
                new_config, http_client=self._http_client
            )

    def _get_default_config(self, client_class: SubClient, path: str):
        config_class = client_class.config_class
        return config_class(
            host=f"{self.base_url}/{path.lstrip('/').rstrip('/')}",
            access_token=self.jwt,
            api_key={"APIKeyHeader": self.api_key} if self.api_key else None,
        )

    async def close(self):
        """Close the client and clean up resources."""
        # close each service
        for service in self._services.values():
            if (
                hasattr(service, "base_client")
                and hasattr(service.base_client, "close")
                and self._owns_http_client
            ):
                await service.base_client.close()
        # close shared http client if we own it
        if self._http_client and self._owns_http_client:
            await self._http_client.close()
            self._http_client = None

    def _ensure_session(self) -> None:
        """
        Lazily create the shared HTTP client when first needed and pass it to all subclients.
        """
        if self._http_client is None:
            self._http_client = ClientSession(
                timeout=ClientTimeout(total=30.0),
                connector=TCPConnector(limit=100, limit_per_host=20),
                headers={"User-Agent": f"crypticorn/python/{self.version}"},
            )
            # Update services to use the new session
            self._services = self._create_services()


class AsyncClient(BaseAsyncClient):
    """
    The official async Python client for interacting with the Crypticorn API.
    It is consisting of multiple microservices covering the whole stack of the Crypticorn project.
    """

    def __init__(
        self,
        *,
        api_key: Optional[str] = None,
        jwt: Optional[str] = None,
        base_url: Optional[str] = None,
        http_client: Optional[ClientSession] = None,
    ):
        """
        :param api_key: The API key to use for authentication (recommended).
        :param jwt: The JWT to use for authentication (not recommended).
        :param base_url: The base URL the client will use to connect to the API.
        :param http_client: The HTTP client to use for the client.
        """
        # Initialize as async client
        super().__init__(
            api_key=api_key,
            jwt=jwt,
            base_url=base_url,
            is_sync=False,
            http_client=http_client,
        )

    async def close(self):
        await super().close()

    async def __aenter__(self):
        self._ensure_session()
        return self

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        await self.close()


class SyncClient(BaseAsyncClient):
    """
    The official synchronous Python client for interacting with the Crypticorn API.
    """

    def __init__(
        self,
        *,
        api_key: Optional[str] = None,
        jwt: Optional[str] = None,
        base_url: Optional[str] = None,
        http_client: Optional[ClientSession] = None,
    ):
        """
        :param http_client: Optional aiohttp ClientSession to use for HTTP requests.
                          Note: For sync client, session management is handled automatically.
        """
        super().__init__(
            api_key=api_key,
            jwt=jwt,
            base_url=base_url,
            is_sync=True,
            http_client=http_client,
        )

    def close(self):
        """Close the client and clean up resources."""
        # For sync client, don't maintain persistent sessions
        # Each operation creates its own session within async_to_sync
        self._http_client = None

    def _ensure_session(self) -> None:
        """
        For sync client, don't create persistent sessions.
        Let each async_to_sync call handle its own session.
        """
        # Don't create persistent sessions in sync mode
        # Each API call will handle session creation/cleanup within async_to_sync
        pass

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.close()

    def __del__(self):
        """Automatic cleanup when the object is garbage collected."""
        try:
            self.close()
        except Exception:
            pass



# coding: utf-8

"""
Notifications API

API for notification delivery system

The version of the OpenAPI document: 1.0.0
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations

import json
import pprint
import re  # noqa: F401
from typing import Any, ClassVar, Dict, List, Optional, Set

from pydantic import (
    BaseModel,
    ConfigDict,
    Field,
    StrictBool,
    StrictStr,
    field_validator,
)
from typing_extensions import Self


class Template(BaseModel):
    """
    Template
    """  # noqa: E501

    identifier: StrictStr = Field(description="Identifier of the template")
    name: StrictStr = Field(description="Name of the template")
    variables: Dict[str, Any] = Field(description="Variables for the template")
    channels: List[StrictStr] = Field(
        description="Channels to send the notification to for this template"
    )
    unsubscribable: StrictBool = Field(
        description="Whether the user can unsubscribe from the template"
    )
    __properties: ClassVar[List[str]] = [
        "identifier",
        "name",
        "variables",
        "channels",
        "unsubscribable",
    ]

    @field_validator("identifier")
    def identifier_validate_enum(cls, value):
        """Validates the enum"""
        if value not in set(
            [
                "subscription_predictions_welcome",
                "subscription_dex_signals_welcome",
                "subscription_combo_welcome",
                "new_member",
                "exchange_api_key_expiring",
                "test",
                "new_dex_ai_call",
                "new_dex_ai_call_incognito",
                "order_completion",
                "trading_agent_execution_alert",
                "otp_code",
                "subscription_expiring",
                "subscription_expired",
            ]
        ):
            raise ValueError(
                "must be one of enum values ('subscription_predictions_welcome', 'subscription_dex_signals_welcome', 'subscription_combo_welcome', 'new_member', 'exchange_api_key_expiring', 'test', 'new_dex_ai_call', 'new_dex_ai_call_incognito', 'order_completion', 'trading_agent_execution_alert', 'otp_code', 'subscription_expiring', 'subscription_expired')"
            )
        return value

    @field_validator("channels")
    def channels_validate_enum(cls, value):
        """Validates the enum"""
        for i in value:
            if i not in set(["email", "telegram", "discord", "websocket", "ui"]):
                raise ValueError(
                    "each list item must be one of ('email', 'telegram', 'discord', 'websocket', 'ui')"
                )
        return value

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of Template from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of Template from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "identifier": obj.get("identifier"),
                "name": obj.get("name"),
                "variables": obj.get("variables"),
                "channels": obj.get("channels"),
                "unsubscribable": obj.get("unsubscribable"),
            }
        )
        return _obj



from crypticorn.auth.client import CreateUser200ResponseAuthAuth as Verify200Response
from crypticorn.auth.client import *
from crypticorn.auth.main import AuthClient

__all__ = [
    "AuthClient",
    "Verify200Response",
]



# coding: utf-8

# flake8: noqa

"""
Notifications API

API for notification delivery system

The version of the OpenAPI document: 1.0.0
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501


__version__ = "1.0.0"

# Define package exports
__all__ = [
    "NotificationsApi",
    "SettingsApi",
    "StatusApi",
    "TemplatesApi",
    "ApiResponse",
    "ApiClient",
    "Configuration",
    "OpenApiException",
    "ApiTypeError",
    "ApiValueError",
    "ApiKeyError",
    "ApiAttributeError",
    "ApiException",
    "Broadcast",
    "BroadcastCreate",
    "BroadcastUpdate",
    "DashboardActionButton",
    "DashboardField",
    "DashboardNotification",
    "DashboardNotificationUpdate",
    "ExceptionDetailLiteralUnknownErrorInvalidDataRequestInvalidDataResponseObjectAlreadyExistsObjectNotFoundTemplateNotFoundInvalidTemplateVariables",
    "NotificationCreate",
    "PaginatedResponseDashboardNotification",
    "Template",
    "UITemplate",
    "UserSetting",
    "UserSettingCreate",
    "UserSettingUpdate",
]

# import apis into sdk package
from crypticorn.notification.client.api.notifications_api import (
    NotificationsApi as NotificationsApi,
)
from crypticorn.notification.client.api.settings_api import SettingsApi as SettingsApi
from crypticorn.notification.client.api.status_api import StatusApi as StatusApi
from crypticorn.notification.client.api.templates_api import (
    TemplatesApi as TemplatesApi,
)
from crypticorn.notification.client.api_client import ApiClient as ApiClient

# import ApiClient
from crypticorn.notification.client.api_response import ApiResponse as ApiResponse
from crypticorn.notification.client.configuration import Configuration as Configuration
from crypticorn.notification.client.exceptions import (
    ApiAttributeError as ApiAttributeError,
)
from crypticorn.notification.client.exceptions import ApiException as ApiException
from crypticorn.notification.client.exceptions import ApiKeyError as ApiKeyError
from crypticorn.notification.client.exceptions import ApiTypeError as ApiTypeError
from crypticorn.notification.client.exceptions import ApiValueError as ApiValueError
from crypticorn.notification.client.exceptions import (
    OpenApiException as OpenApiException,
)

# import models into sdk package
from crypticorn.notification.client.models.broadcast import Broadcast as Broadcast
from crypticorn.notification.client.models.broadcast_create import (
    BroadcastCreate as BroadcastCreate,
)
from crypticorn.notification.client.models.broadcast_update import (
    BroadcastUpdate as BroadcastUpdate,
)
from crypticorn.notification.client.models.dashboard_action_button import (
    DashboardActionButton as DashboardActionButton,
)
from crypticorn.notification.client.models.dashboard_field import (
    DashboardField as DashboardField,
)
from crypticorn.notification.client.models.dashboard_notification import (
    DashboardNotification as DashboardNotification,
)
from crypticorn.notification.client.models.dashboard_notification_update import (
    DashboardNotificationUpdate as DashboardNotificationUpdate,
)
from crypticorn.notification.client.models.exception_detail_literal_unknown_error_invalid_data_request_invalid_data_response_object_already_exists_object_not_found_template_not_found_invalid_template_variables import (
    ExceptionDetailLiteralUnknownErrorInvalidDataRequestInvalidDataResponseObjectAlreadyExistsObjectNotFoundTemplateNotFoundInvalidTemplateVariables as ExceptionDetailLiteralUnknownErrorInvalidDataRequestInvalidDataResponseObjectAlreadyExistsObjectNotFoundTemplateNotFoundInvalidTemplateVariables,
)
from crypticorn.notification.client.models.notification_create import (
    NotificationCreate as NotificationCreate,
)
from crypticorn.notification.client.models.paginated_response_dashboard_notification import (
    PaginatedResponseDashboardNotification as PaginatedResponseDashboardNotification,
)
from crypticorn.notification.client.models.template import Template as Template
from crypticorn.notification.client.models.ui_template import UITemplate as UITemplate
from crypticorn.notification.client.models.user_setting import (
    UserSetting as UserSetting,
)
from crypticorn.notification.client.models.user_setting_create import (
    UserSettingCreate as UserSettingCreate,
)
from crypticorn.notification.client.models.user_setting_update import (
    UserSettingUpdate as UserSettingUpdate,
)
